## Overview

[dom_shim.mjs](../dom_shim.mjs) provides a lightweight and performant shim for DOM node and element classes. Allows server-side rendering with [`prax`](prax_readme.md).

## TOC

* [#Usage](#usage)
* [#API](#api)
  * [#Undocumented](#undocumented)

## Usage

```js
import * as ds from 'https://cdn.jsdelivr.net/npm/@mitranim/js@0.1.67/dom_shim.mjs'
```

## API

### Undocumented

The following APIs are exported but undocumented. Check [dom_shim.mjs](../dom_shim.mjs).

  * [`const SYM_REF`](../dom_shim.mjs#L10)
  * [`const SYM_DATA`](../dom_shim.mjs#L11)
  * [`const SYM_NAME`](../dom_shim.mjs#L12)
  * [`const SYM_VALUE`](../dom_shim.mjs#L13)
  * [`const SYM_STYLE`](../dom_shim.mjs#L14)
  * [`const SYM_DOCTYPE`](../dom_shim.mjs#L15)
  * [`const SYM_DATASET`](../dom_shim.mjs#L16)
  * [`const SYM_PUBLIC_ID`](../dom_shim.mjs#L17)
  * [`const SYM_SYSTEM_ID`](../dom_shim.mjs#L18)
  * [`const SYM_CLASS_LIST`](../dom_shim.mjs#L19)
  * [`const SYM_LOCAL_NAME`](../dom_shim.mjs#L20)
  * [`const SYM_ATTRIBUTES`](../dom_shim.mjs#L21)
  * [`const SYM_PARENT_NODE`](../dom_shim.mjs#L22)
  * [`const SYM_CHILD_NODES`](../dom_shim.mjs#L23)
  * [`const SYM_NAMESPACE_URI`](../dom_shim.mjs#L24)
  * [`const SYM_OWNER_DOCUMENT`](../dom_shim.mjs#L25)
  * [`const SYM_IMPLEMENTATION`](../dom_shim.mjs#L26)
  * [`const SYM_DOCUMENT_ELEMENT`](../dom_shim.mjs#L27)
  * [`function isChildNode`](../dom_shim.mjs#L36)
  * [`function reqChildNode`](../dom_shim.mjs#L37)
  * [`function optChildNode`](../dom_shim.mjs#L38)
  * [`function isParentNode`](../dom_shim.mjs#L40)
  * [`function reqParentNode`](../dom_shim.mjs#L41)
  * [`function optParentNode`](../dom_shim.mjs#L42)
  * [`function isElement`](../dom_shim.mjs#L44)
  * [`function reqElement`](../dom_shim.mjs#L45)
  * [`function optElement`](../dom_shim.mjs#L46)
  * [`function isAttr`](../dom_shim.mjs#L48)
  * [`function reqAttr`](../dom_shim.mjs#L49)
  * [`function optAttr`](../dom_shim.mjs#L50)
  * [`function isText`](../dom_shim.mjs#L52)
  * [`function reqText`](../dom_shim.mjs#L53)
  * [`function optText`](../dom_shim.mjs#L54)
  * [`function isComment`](../dom_shim.mjs#L56)
  * [`function reqComment`](../dom_shim.mjs#L57)
  * [`function optComment`](../dom_shim.mjs#L58)
  * [`function isDocument`](../dom_shim.mjs#L60)
  * [`function reqDocument`](../dom_shim.mjs#L61)
  * [`function optDocument`](../dom_shim.mjs#L62)
  * [`function isDocumentType`](../dom_shim.mjs#L64)
  * [`function reqDocumentType`](../dom_shim.mjs#L65)
  * [`function optDocumentType`](../dom_shim.mjs#L66)
  * [`function isFragment`](../dom_shim.mjs#L68)
  * [`function reqFragment`](../dom_shim.mjs#L69)
  * [`function optFragment`](../dom_shim.mjs#L70)
  * [`function isDomImpl`](../dom_shim.mjs#L72)
  * [`function reqDomImpl`](../dom_shim.mjs#L73)
  * [`function optDomImpl`](../dom_shim.mjs#L74)
  * [`class Node`](../dom_shim.mjs#L78)
  * [`class Textable`](../dom_shim.mjs#L232)
  * [`class TextableElementParent`](../dom_shim.mjs#L254)
  * [`class DocumentFragment`](../dom_shim.mjs#L261)
  * [`class Void`](../dom_shim.mjs#L267)
  * [`class Data`](../dom_shim.mjs#L276)
  * [`class CharacterData`](../dom_shim.mjs#L282)
  * [`class Text`](../dom_shim.mjs#L296)
  * [`class Comment`](../dom_shim.mjs#L303)
  * [`class Attr`](../dom_shim.mjs#L315)
  * [`class NamedNodeMap`](../dom_shim.mjs#L325)
  * [`class Element`](../dom_shim.mjs#L362)
  * [`class HTMLElement`](../dom_shim.mjs#L581)
  * [`class HTMLMetaElement`](../dom_shim.mjs#L596)
  * [`class HTMLAnchorElement`](../dom_shim.mjs#L601)
  * [`class HTMLLabelElement`](../dom_shim.mjs#L623)
  * [`class HTMLButtonElement`](../dom_shim.mjs#L633)
  * [`class HTMLInputElement`](../dom_shim.mjs#L658)
  * [`class HTMLTextAreaElement`](../dom_shim.mjs#L670)
  * [`class HTMLObjectElement`](../dom_shim.mjs#L672)
  * [`class HTMLOutputElement`](../dom_shim.mjs#L674)
  * [`class HTMLSelectElement`](../dom_shim.mjs#L680)
  * [`class HTMLOptionElement`](../dom_shim.mjs#L688)
  * [`class HTMLFieldSetElement`](../dom_shim.mjs#L696)
  * [`class HTMLFormElement`](../dom_shim.mjs#L698)
  * [`class HTMLTableElement`](../dom_shim.mjs#L702)
  * [`class HTMLScriptElement`](../dom_shim.mjs#L711)
  * [`class SVGElement`](../dom_shim.mjs#L729)
  * [`class DocumentType`](../dom_shim.mjs#L734)
  * [`class Document`](../dom_shim.mjs#L758)
  * [`class HTMLDocument`](../dom_shim.mjs#L849)
  * [`class DOMImplementation`](../dom_shim.mjs#L866)
  * [`class CustomElementRegistry`](../dom_shim.mjs#L898)
  * [`class DictPh`](../dom_shim.mjs#L925)
  * [`class StylePh`](../dom_shim.mjs#L938)
  * [`class ClassList`](../dom_shim.mjs#L1057)
  * [`class HTMLFormControlsCollection`](../dom_shim.mjs#L1153)
  * [`class FormControlMap`](../dom_shim.mjs#L1216)
  * [`class RadioNodeList`](../dom_shim.mjs#L1239)
  * [`class GlobPh`](../dom_shim.mjs#L1245)
  * [`const global`](../dom_shim.mjs#L1264)
  * [`const document`](../dom_shim.mjs#L1265)
  * [`const customElements`](../dom_shim.mjs#L1266)
  * [`function escapeAttr`](../dom_shim.mjs#L1276)
  * [`function escapeText`](../dom_shim.mjs#L1289)
  * [`function escapeChar`](../dom_shim.mjs#L1296)
  * [`function unescape`](../dom_shim.mjs#L1305)
  * [`function unescapeMatch`](../dom_shim.mjs#L1309)
  * [`const styleToCssCache`](../dom_shim.mjs#L1324)
  * [`const camelToDataCache`](../dom_shim.mjs#L1328)
  * [`const dataToCamelCache`](../dom_shim.mjs#L1332)
  * [`const OUTER_HTML`](../dom_shim.mjs#L1337)
